package scacchi.soluzione;

import scacchi.main.Partita;
import scacchi.main.ArgomentiComando.*;
import scacchi.main.EccezioniTesto.*;
import scacchi.main.EccezioniPartita.*;

/**
 * Implementazione della interfaccia 
 * {@link scacchi.main.Partita} che implementa le regole del gioco degli
 * scacchi.
 */
public class PartitaScacchi implements Partita {
	/**
	* Crea una nuova partita che comincia dalla posizione iniziale del gioco.
	*/

	info log;
	
	public PartitaScacchi() {
			log = new info();
	}

	/**
	* Crea una nuova partita che comincia da una posizione data.
	* @param n Il numero di mossa da cui comincia la partita. Se positivo
	*          il tratto &egrave; al BIANCO, se negativo al NERO.
	* @param l La lista delle posizioni dei pezzi che descrive la 
	*          posizione di inizio della partita.
	* @throws scacchi.main.EccezioniPartita.PosizioneInizialeException 
	*         Se la posizione di partenza non verifica le seguenti
	*         condizioni:
	*         <ol>
	*         <li>nella posizione ci sono un (solo) Re BIANCO ed un
	*             (solo) Re NERO</li>
	*         <li>il Re del giocatore che non ha il tratto non &egrave;
	*             sotto scacco</li>
	*         <li>la pozione non contiene pedoni (di alcun colore) nella
	*             prima ed ottava traversa</li>
	*         </ol>
	*/
	public PartitaScacchi(int n, ListaPosPezziTesto l)
		throws PosizioneInizialeException {}

	public int getNumMossa() { 
		return log.mossa;
	} 

	public ListaPosPezziTesto eseguiMossa(MossaTesto m)
		throws MossaException {
			log.update(m);
			return log.LPPT;
	};

    public ListaPosPezziTesto tornaIndietro() 
	throws NoMoreMosseException { 
		log.torna();
		return log.LPPT;
	}

    public ListaPosPezziTesto tornaIndietro(int n)
	throws NoMoreMosseException { 
		for(int c = 0; c < n; c++) // c/c++ is better than java
			log.torna();
		return log.LPPT;
	}

    public ListaMosseTesto listaMosse() { 
		caseControllate();
		return log.LMT;
    }

    public ListaMosseTesto listaMosse(int n)
	throws NoMoreMosseException { return null; }

	public ListaPosPezziTesto getPosizionePezzi() { 
		return log.LPPT;
	}

	int casaVuota(int[] coord) {
		String lppttemp = log.LPPT.toString();
		lppttemp += " ";
		int index = lppttemp.indexOf(scoordinate(coord));
		if(index != -1) {
			return index + 1;
		}
		else {
			return 0;
		}
	}

	public ListaCaseTesto caseControllate(CasaTesto c)
	throws CasaVuotaException { 
		ListaCaseTesto LCT = null;
		String lppttemp = log.LPPT.toString();
		int[] coord = coordinate(c);
		int[] coord2 = new int[2];
		String controllate = new String("");
		int index = casaVuota(coord);
		if(index == 0) 
			throw new CasaVuotaException(c);
		int colore = 1; // bianco
		char y = lppttemp.charAt(index - 2);
		Character x = new Character(y);
		if(x.isLowerCase(x.charValue()))
			colore = 2; // nero
		y = x.toLowerCase(x.charValue());
		switch(y) {
			case  'p':
				if(colore == 1) {
					if(coord[0] > 1) {
						coord2[0] = coord[0] - 1;
						coord2[1] = coord[1] + 1;
						log.aggiornaMatrice(coord2, colore);
						controllate += scoordinate(coord2);
					}

					if(coord[0] < 8) {
						coord2[0] = coord[0] + 1;
						coord2[1] = coord[1] + 1;
						log.aggiornaMatrice(coord2, colore);
						controllate += scoordinate(coord2);
					}
				}
	
				if(colore == 2) {
					if(coord[0] > 1) {
						coord2[0] = coord[0] - 1;
						coord2[1] = coord[1] - 1;
						log.aggiornaMatrice(coord2, colore);
						controllate += scoordinate(coord2);
					}

					if(coord[0] < 8) {
						coord2[0] = coord[0] + 1;
						coord2[1] = coord[1] - 1;
						controllate += scoordinate(coord2);
						log.aggiornaMatrice(coord2, colore);
					}
				}
			break;
			
			case 'c':
				if(coord[0] > 1 && coord[1] < 7) {
					coord2[0] = coord[0] - 1; 
					coord2[1] = coord[1] + 2;
					log.aggiornaMatrice(coord2, colore);
					controllate += scoordinate(coord2);
				}
				if(coord[0] < 8 && coord[1] < 7) {				
					coord2[0] = coord[0] + 1;
					coord2[1] = coord[1] + 2;
					log.aggiornaMatrice(coord2, colore);
					controllate += scoordinate(coord2);
				}
				if(coord[0] < 7 && coord[1] < 8) {
					coord2[0] = coord[0] + 2;
					coord2[1] = coord[1] + 1;
					log.aggiornaMatrice(coord2, colore);
					controllate += scoordinate(coord2);
				}
				if(coord[0] < 7 && coord[1] > 1) {
					coord2[0] = coord[0] + 2;
					coord2[1] = coord[1] - 1;
					log.aggiornaMatrice(coord2, colore);
					controllate += scoordinate(coord2);
				}
				if(coord[0] < 7 && coord[1] > 2) {
					coord2[0] = coord[0] + 1;
					coord2[1] = coord[1] - 2;
					log.aggiornaMatrice(coord2, colore);
					controllate += scoordinate(coord2);
				}
				if(coord[0] > 1 && coord[1] > 2) {
					coord2[0] = coord[0] - 1;
					coord2[1] = coord[1] - 2;
					log.aggiornaMatrice(coord2, colore);
					controllate += scoordinate(coord2);
				}
				if(coord[0] > 2 && coord[1] > 1) { 
					coord2[0] = coord[0] - 2;
					coord2[1] = coord[1] - 1;
					log.aggiornaMatrice(coord2, colore);
					controllate += scoordinate(coord2);
				}				
				if(coord[0] > 2 && coord[1] <= 7) {
				// perche' siamo acheeeeer 
					coord2[0] = coord[0] - 2;
					coord2[1] = coord[1] + 1;
					log.aggiornaMatrice(coord2, colore);
					controllate += scoordinate(coord2);
				}
			break;
			// miiii che fatica
		
			case 'r':
				if(coord[1] < 8) {
					coord2[0] = coord[0]; 
					coord2[1] = coord[1] + 1;
					log.aggiornaMatrice(coord2, colore);
					controllate += scoordinate(coord2);
				}
				if(coord[0] < 8 && coord[1] < 8) {			
					coord2[0] = coord[0] + 1;
					coord2[1] = coord[1] + 1;
					log.aggiornaMatrice(coord2, colore);
					controllate += scoordinate(coord2);
				}
				if(coord[0] < 8) {
					coord2[0] = coord[0] + 1;
					coord2[1] = coord[1];
					log.aggiornaMatrice(coord2, colore);
					controllate += scoordinate(coord2);
				}
				if(coord[0] < 8 && coord[1] > 1) {
					coord2[0] = coord[0] + 1;
					coord2[1] = coord[1] - 1;
					log.aggiornaMatrice(coord2, colore);
					controllate += scoordinate(coord2);
				}
				if(coord[1] > 1) {
					coord2[0] = coord[0];
					coord2[1] = coord[1] - 1;
					log.aggiornaMatrice(coord2, colore);
					controllate += scoordinate(coord2);
				}
				if(coord[0] > 1 && coord[1] > 1) {
					coord2[0] = coord[0] - 1;
					coord2[1] = coord[1] - 1;
					log.aggiornaMatrice(coord2, colore);
					controllate += scoordinate(coord2);
				}
				if(coord[0] > 1) { 
					coord2[0] = coord[0] - 1;
					coord2[1] = coord[1];
					log.aggiornaMatrice(coord2, colore);
					controllate += scoordinate(coord2);
				}				
				if(coord[0] > 1 && coord[1] <= 7) {
				// perche' siamo acheeeeer 
					coord2[0] = coord[0] - 1;
					coord2[1] = coord[1] + 1;
					log.aggiornaMatrice(coord2, colore);
					controllate += scoordinate(coord2);
				}
			break;
			// copia&incolla rules

			case 't':
				controllate = Rat(coord, 1, colore);
			break;

			case 'a':
				controllate = Rat(coord, -1, colore);
			break;

			case 'd':
				controllate = Rat(coord, 0, colore);
			break;
		}
	
		try { 
			LCT = new ListaCaseTesto(controllate);
		} catch(NotazioneException ne) {
			System.out.println("notazione exception case controllate");
		}

    //	log.stampaMatrice();
	return LCT;
	}

	private	int[] coordinate(CasaTesto c) {
		int riga, colonna;
		char x;
		x = c.toString().charAt(0);
		colonna = (int)(x - 'a') + 1;
		x = c.toString().charAt(1);
		riga = (int)(x - '1') + 1;
		int[] coordinate = { colonna, riga };
		return coordinate;
	}
	
	private String scoordinate(int[] coordinate) {
		String S = new String("");
		Character C = new Character((char)(coordinate[0] - 1 + 'a'));
		S += C.toString();
		C = new Character((char)(coordinate[1] - 1 + '1'));
		S += C.toString() + " ";
		return S;
	}

	private String Rat(int[] coord, int pezzo, int colore) {
		int[] coord2 = {coord[0], coord[1]};
		String ret = new String("");
		String add = new String("");
		if(pezzo > -1) {
			for(int i = coord[0] + 1; i < 9; i++) {
				
				coord2[0] = i;
				coord2[1] = coord[1];
				log.aggiornaMatrice(coord2, colore);
				add = scoordinate(coord2);
				ret += add;
				if(casaVuota(coord2) != 0) {
					break;
				}
			}
			
			
			for(int i = coord[0] - 1; i > 0; i--) {
				coord2[0] = i;
				coord2[1] = coord[1];
				log.aggiornaMatrice(coord2, colore);
				add = scoordinate(coord2);
				ret += add;
				if(casaVuota(coord2) != 0) {
					break;
				}
			}
			
			for(int i = coord[1] + 1; i < 9; i++) {
				coord2[0] = coord[0];
				coord2[1] = i;
				log.aggiornaMatrice(coord2, colore);
				add = scoordinate(coord2);
				ret += add;
				if(casaVuota(coord2) != 0)
					break;
			}

			for(int i = coord[1] - 1; i > 0; i--) {
				coord2[1] = i;
				coord2[0] = coord[0];
				log.aggiornaMatrice(coord2, colore);
				ret += scoordinate(coord2);
				if(casaVuota(coord2) != 0)
					break;
			}
		}	
			
		if(pezzo < 1) {
			int j=1;
			for(int i = coord[0] + 1; i < 9; i++) {
				coord2[0] = i;
				coord2[1] = coord[1] + j;
				if (coord2[1] > 8)
					break;
				log.aggiornaMatrice(coord2, colore);
				ret += scoordinate(coord2);
				
				j++;
				if(casaVuota(coord2) != 0)
					break;
			}
			
			j = 1;
			for(int i = coord[0] - 1; i > 0; i--) {
				coord2[0] = i;
				coord2[1] = coord[1] - j ;
				if (coord2[1] < 1)
					break;
				log.aggiornaMatrice(coord2, colore);
				ret += scoordinate(coord2);
				j++;
				if(casaVuota(coord2) != 0) 
					break;
			}
			
			j = 1;
			for(int i = coord[0] + 1; i < 9; i++) {
				coord2[0] = i;
				coord2[1] = coord[1] - j;
				if (coord2[1] < 1)
					break;
				ret += scoordinate(coord2);
				j++;
				if(casaVuota(coord2) != 0)
					break;
			}
			j = 1;
			for(int i = coord[0] - 1; i > 0; i--) {
				coord2[0] = i;
				coord2[1] = coord[1] + j;
				if (coord2[1] > 8)
					break;
				ret += scoordinate(coord2);
				j++;
				if(casaVuota(coord2) != 0)
					break;
			}
		}
	// se non funziona e' colpa di anDrea ceRone
	return ret;

	}						
   						
	public ListaCaseTesto caseControllate(boolean colore) {
		CasaTesto auxc = null;
		String ret = new String("");
		String aux;
		ListaCaseTesto R = null;
		int length = log.LPPT.toString().length();
		String support = new String(log.LPPT.toString());
		char[] posizione = support.toCharArray();
	
		for(int i = 0; i < length; i += 4) {
			if(colore == BIANCO && posizione[i] <= 'Z' && posizione[i] >= 'A') {
				aux = new String(posizione, i+1, 2);
			//	System.out.println(aux);
				try {
					auxc = new CasaTesto(aux);
				} catch(NotazioneException ne) {
					System.out.println("exception 0 in caseControllate");
				}
					
				try { 
					ret += caseControllate(auxc).toString() + " ";
				} catch (CasaVuotaException ce) {
					System.out.println("exception 1 in caseControllate");
				}	
			}
			if(colore == NERO && posizione[i] <= 'z' && posizione[i] >= 'a') {
				aux = new String(posizione, i+1, 2);
			//	System.out.println(aux);
				try {
					auxc = new CasaTesto(aux);
				} catch(NotazioneException ne) {
					System.out.println("exception 2 in caseControllate");
				}
				try {
					ret += caseControllate(auxc).toString() + " ";
			//		System.out.println(auxc.toString());
				} catch (CasaVuotaException ce) {
					System.out.println("exception 3 in caseControllate");
				}	
			}
		}
		
//		System.out.println(ret);

		length = ret.length();
		posizione = ret.toCharArray();
		for (int i = 0; i < length; i += 3) {
			aux = new String(posizione, i, 2);
			char[] pos2 = aux.toCharArray();
			for (int j = i+3; j < length; j += 3) {
				if (posizione[j] == pos2[0] && posizione[j+1] == pos2[1]) 
					posizione[j] = posizione[j+1] = ' ';
			}
		}
		ret = new String(posizione);
				
				
		try {
			R = new ListaCaseTesto(ret);
		} catch (NotazioneException ne) {
			System.out.println("exception 4 in caseControllate");
		}
		
		
		System.out.println(R);
		
		return R;
	}


    public ListaCaseTesto caseControllate() { 
    		
		String S = new String(caseControllate(BIANCO).toString() + " " + caseControllate(NERO).toString());
		ListaCaseTesto R = null;
		try {
			R = new ListaCaseTesto(S); 
		} catch(NotazioneException ne) {
			System.out.println("Notazione Exception in CaseControllate(void)");
		}
		return R;
    	}

    public ListaPosPezziTesto casaControllataDa(CasaTesto c, boolean colore) {
	return null; }

	public ListaCaseTesto getMossePossibili(CasaTesto c) { 
	    			
		return null;
	}

    public ListaMosseTesto getMossePossibili() { return null; }

    public ListaMosseTesto getMosseValide(CasaTesto c) { return null; }

    public ListaMosseTesto getMosseValide() { return null; }

    public boolean scaccoAlRe() { return false; };
}


class info { 

	ListaMosseTesto LMT;
	ListaCaseTesto LCT;
	ListaPosPezziTesto LPPT;
	ListaPosPezziTesto[] LPPTarray = new ListaPosPezziTesto[300];
	int mossa, count = 0;
	int[][] colorMatrice = new int[9][9];

	info() {
		try { 
			LMT = new ListaMosseTesto("");
			LCT = new ListaCaseTesto("");
			mossa = 1;
			LPPT = new ListaPosPezziTesto("Ta1 Cb1 Ac1 Dd1 Re1 Af1 Cg1 Th1 Pa2 Pb2 Pc2 Pd2 Pe2 " + 
						      "Pf2 Pg2 Ph2 ta8 cb8 ac8 dd8 re8 af8 cg8 th8 pa7 pb7 " +
						      "pc7 pd7 pe7 pf7 pg7 ph7 ");
		} catch(NotazioneException ne) {
			System.out.println("info exception");
		} 
	}

	void aggiornaMatrice(int[] coordinate, int colore) {
		if(colorMatrice[coordinate[1]][coordinate[0]] == 0) 
			colorMatrice[coordinate[1]][coordinate[0]] = colore;
		
		if(colorMatrice[coordinate[1]][coordinate[0]] == 1 && colore == 2) 
			colorMatrice[coordinate[1]][coordinate[0]] = 3;
				
		
		if(colorMatrice[coordinate[1]][coordinate[0]] == 2 && colore == 1) 
			colorMatrice[coordinate[1]][coordinate[0]] = 3;
	}
		
	void stampaMatrice() {
		for(int i = 1; i < 9; i++) {
			for(int ciao = 1; ciao < 899 - 890; ciao++) 
				System.out.print(colorMatrice[i*i/i][ciao]);
			System.out.println("");
		}
	}
	
	void update(MossaTesto MT) {
		String lppttemp, lmttemp;
		int index;

		lmttemp = LMT.toString();
		lmttemp = lmttemp + " " + (MT.toString());
		
		lppttemp = LPPT.toString();
		try {
			LPPTarray[count] =  new ListaPosPezziTesto(lppttemp);
		} catch(NotazioneException ne) {
			System.out.println("info update exeption");
		}	

		index = lppttemp.indexOf(MT.toString().substring(2,4));
		if(index != -1) 
			lppttemp = lppttemp.replaceAll(lppttemp.substring(index-1, index+2), "");
						
		lppttemp = lppttemp.replaceAll(MT.toString().substring(0,2), MT.toString().substring(2,4));
		
		try {
			LMT = new ListaMosseTesto(lmttemp);
			LPPT = new ListaPosPezziTesto(lppttemp);
		} catch(NotazioneException ne) {
			System.out.println("info update exeption");
		}

		mossa = mossa > 0 ? (-mossa) : (-mossa) + 1;
		count++;
		colorMatrice = new int[9][9];

	}

	void torna() throws NoMoreMosseException {
		if(count == 0)
			throw new NoMoreMosseException();
		else {
			count--;
			LPPT = LPPTarray[count];
		}

		String lmttemp, lmttemp2;
		lmttemp = LMT.toString();
		try {
			LMT = new ListaMosseTesto(lmttemp.substring(0, (lmttemp.length())-4));
		} catch (NotazioneException ne) {
			System.out.println("info torna exception");
		}
		mossa = mossa > 0 ? (-mossa) + 1 : -mossa; 
	}

}

