package scacchi.soluzione;

import scacchi.main.Partita;
import scacchi.main.ArgomentiComando.*;
import scacchi.main.EccezioniTesto.*;
import scacchi.main.EccezioniPartita.*;

/**
	Classe con tutti metodi statici per lavorare su CasaTesto, ListaPosPezziTesto, MossaTesto
	
*/

public class MetodiStringa
{
	/**
		Estrae dalla lista l, il pezzo nella casella c
		@param c la casella dove si trova il pezzo
		@param l la lista con la posizione di tutti i pezzi
		@return il carattere rappresentante il pezzo voluto
	*/
	static char GetPezzo(CasaTesto c, ListaPosPezziTesto l)
	{
		String temp=null;
		int i;
		temp=l.toString();
		i= temp.indexOf(c.toString());
		if(i!=-1)
			return temp.charAt(i-1);
		else
			return 'e'; // errore
	
	}
	/**
		Estrae dalla lista l, il pezzo interessato dalla mossa m
		@param m mossa che deve fare il pezzo
		@param l la lista con la posizione di tutti i pezzi
		@return il carattere rappresentante il pezzo voluto
	*/
	static char GetPezzo(MossaTesto m, ListaPosPezziTesto l)
	{
		String lista, pos;
		int i;
		lista=l.toString();
		pos=new String(m.toString().toCharArray(),0,2);
		i=lista.indexOf(pos);
		if(i!=-1)
			return lista.charAt(i-1);
		else 
			return 'e';//errore
	}
	/**
		Restituisce un intero che rappresenta il pezzo voluto, questo intero serve per sapere i movimenti che il pezzo pu fare
		@param pezzo il carattere rappresentante il pezzo
		@return un intero che rappresenta il pezzo voluto, questo intero serve per sapere i movimenti che il pezzo pu fare. Il range  -1,6 
		dove -1 rappresenta un errore
		@see PartitaScacchi#pezzi
	*/
	static int GetPezzo(char pezzo)
	{
		switch (pezzo){
			case 'T':
				return 0;
			case 'C':
				return 1;
			case 'A':
				return 2;
			case 'D':
				return 3;
			case 'R':
				return 4;
			case 'P':
				return 5;
			case 't':
				return 0;
			case 'c':
				return 1;
			case 'a':
				return 2;
			case 'd':
				return 3;
			case 'r':
				return 4;
			case 'p':
				return 6;
			default:
				return -1;
		}
		
	}
	/**
		Ottiene il numero di riga dalla posizione in CasaTesto c
		@param c posizione CasaTesto della quale si vuole sapere la riga
		@return un intero rappresentante le righe della scacchiera
	*/
	static int getRiga(CasaTesto c)
	{	
		char r=c.toString().charAt(1);
		switch(r){
			case '1':
				return 1;
			case '2':
				return 2;
			case '3':
				return 3;
			case '4':
				return 4;
			case '5':
				return 5;
			case '6':
				return 6;
			case '7':
				return 7;
			case '8':
				return 8;
			default:
				return -1;
		}
	}
	/**
		Ottiene il numero di colonna dalla posizione in CasaTesto c
		@param c posizione CasaTesto della quale si vuole sapere la colonna
		@return un intero rappresentante la colonna della scacchiera
	*/
	static int getColonna(CasaTesto c)
	{
		char col=c.toString().charAt(0);
		switch(col){
			case 'a':
				return 1;
			case 'b':
				return 2;
			case 'c':
				return 3;
			case 'd':
				return 4;
			case 'e':
				return 5;
			case 'f':
				return 6;
			case 'g':
				return 7;
			case 'h':
				return 8;
			default:
				return -1;
		}
	}
	/**
		Ottiene il colore di un pezzo
		@param pezzo il carattere rappresentante il pezzo interessato
		@return true se Bianco, false altrimenti
	*/
	static boolean getColore(char pezzo)
	{
		switch(pezzo){
		case 'T':
		case 'C':
		case 'A':
		case 'D':
		case 'R':
		case 'P':
			return true; //bianco
		default:
			return false;
		}
	}
	/**
		Cerca nella stringa d le occorrenze delle sottostringhe ottenute da s, usando gli spazi come delimitatori:
		se non ci sono aggiunge a d le sottostringhe
		@param s stringa contente sottostringhe separate da spazi
		@param d stringa contente sottostringhe separate da spazi, nella quale si aggiungeranno le sottostringhe di s che non sono gi
		presenti
		@return la stringa ottenuta unendo s e d senza doppioni di sottostringhe
	*/
	static String inserisciStringa(String s, String d)
	{
		String test="";
		int r=1;
		if(d.length()<=0)
			return s;
		// spezzo s in tante sottostringhe divise dagli spazi
		r=s.indexOf(" ");
		while(r!=-1)
		{
			test=s.substring(0,r);
			s=s.substring(r+1);
			if(d.indexOf(test) ==-1)
			{
				d=d+" "+test;
			}
			r=s.indexOf(" ");
		}
		if(d.indexOf(s) ==-1)
			{
				d=d+" "+s;
			}
		return d;
	
	
	}

} 
